package com.hzya.frame.u8c.sosale.service.impl;

import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.basedao.service.impl.BaseService;
import com.hzya.frame.bip.v3.v2207.util.BipUtil;
import com.hzya.frame.seeyon.util.RestUtil;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.service.IIntegrationTaskLivingDetailsService;
import com.hzya.frame.u8c.sosale.dao.ISoSaleDao;
import com.hzya.frame.u8c.sosale.dao.ISoSaleLineDao;
import com.hzya.frame.u8c.sosale.entity.SoSaleEntity;
import com.hzya.frame.u8c.sosale.entity.SoSaleLineEntity;
import com.hzya.frame.u8c.sosale.service.ISoSaleService;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

@Service(value = "SoSaleServiceImpl")
public class SoSaleServiceImpl extends BaseService<SoSaleEntity, String> implements ISoSaleService {
    Logger logger = LoggerFactory.getLogger(SoSaleServiceImpl.class);
    @Autowired
    private ISoSaleDao u8cSaleDao;
    @Autowired
    private ISoSaleLineDao saleLineDao;
    @Autowired
    private IIntegrationTaskLivingDetailsService taskLivingDetailsService;

    /**
     *
     * @content 获取U8C销售订单同步OA销售合同
     * @className: Administrator
     * @author laborer
     * @date 2024-10-16 15:22
     *
     */
    @Override
    public JsonResultEntity getU8cSaleToOa(JSONObject requestJson) {

        //获取重推信息id，并记录日志
        JSONObject requestJsonObj = JSON.parseObject(requestJson.toString(),JSONObject.class);
        String interId = requestJsonObj.getString("integration_task_living_details_id");
        logger.info("U8C销售订单推送OA重推解析后："+requestJsonObj.toString());
        logger.info("U8C销售订单推送OA重新执行的任务主键："+interId);

        //更换数据源
        JsonResultEntity resultEntity = new JsonResultEntity();
        SoSaleEntity entity = new SoSaleEntity();
        requestJson.put("db_code","dongjinU8C");
        entity.setDataSourceCode(requestJson.getString("db_code"));

        //根据重推信息id，查询该条重推记录信息
        IntegrationTaskLivingDetailsEntity oldMsg = new IntegrationTaskLivingDetailsEntity();
        if(StrUtil.isNotEmpty(interId)){
            oldMsg = taskLivingDetailsService.get(interId);
        }
        String rootAppPk = oldMsg.getRootAppPk();
        entity.setId(rootAppPk);
        StrUtil.isEmpty(rootAppPk);//entity.setQueryState("查询");
//如果是重推，是否通过entity传入的id，只能查到一条数据。
        //entity.setId("-8845102890648320822");
        List<SoSaleEntity> soleList = u8cSaleDao.getU8cSale(entity);
        logger.info("销售订单查询数据{}",JSONObject.toJSONString(soleList));
        if(CollectionUtils.isNotEmpty(soleList)){
            soleList.forEach(sale -> {
                try{
                    //获取明细数据进行拼接
                    String csaleId = sale.getCsaleId();
                    if(!StrUtil.isEmpty(csaleId)){
                        SoSaleLineEntity line = new SoSaleLineEntity();
                        line.setCsaleId(csaleId);
                        line.setDataSourceCode(requestJson.getString("db_code"));
                        List<SoSaleLineEntity> lineList = saleLineDao.getU8cSaleLine(line);
                        logger.info("销售订单明细查询数据{}",JSONObject.toJSONString(lineList));
                        sale.setSaleLineList(lineList);
                        logger.info("组装数据");
                        JSONObject main = bindingAdd(sale);
                        logger.info("销售订单推送数据："+main.toString());
                        String result = RestUtil.sendU8CToOA(main.toString(),"8000340003",sale.getUser_code());
                        logger.info("销售订单推送结果："+result);
                        IntegrationTaskLivingDetailsEntity logDetails = new IntegrationTaskLivingDetailsEntity();
                        logDetails.setRootAppPk(sale.getCsaleId());
                        logDetails.setRootAppBill(sale.getBillCode());
                        logDetails.setNewTransmitInfo(result);
                        logDetails.setNewPushDate(new Date());
                        logDetails.setRootAppNewData(JSON.toJSONString(sale));
                        logDetails.setPluginId("SaleConPlugin");

                        JSONObject resultObj = JSON.parseObject(result);
                        boolean flag = resultObj.getString("code").equals("0");
                        if(flag){
                            // 推送成功，更新状态
                            sale.setState("Y");
                        }else{
                            // 推送失败
                            sale.setState("N");
                        }
                        sale.setDataSourceCode(requestJson.getString("db_code"));
                        try{
                            if(StrUtil.isEmpty(interId)){
                                if(flag){
                                    //首推成功
                                    taskLivingDetailsService.saveLogToSuccess(logDetails);
                                }else{
                                    //首推失败
                                    taskLivingDetailsService.saveLogToFail(logDetails);
                                }
                            }else{
                                logDetails.setId(interId);
                                if(flag){
                                    //重推成功
                                    taskLivingDetailsService.saveLogFailToSuccess(logDetails);
                                }else{
                                    //重推失败，把失败信息，更新到重推表中
                                    taskLivingDetailsService.updateLogFailToSuccess(logDetails);
                                }
                            }
                        }catch (Exception e){
                            logger.info("保存日志失败"+e.getMessage());
                            u8cSaleDao.updateState(sale);
                            e.printStackTrace();
                        }
                        u8cSaleDao.updateState(sale);
                    }
                } catch (Exception e) {
                    logger.info("销售订单执行失败"+e.getMessage());
                    e.printStackTrace();
                }
            });

        }
        return new JsonResultEntity("成功",true,resultEntity);
    }
/**
 *
 * @content 组装销售订单数据
 * @className: Administrator
 * @author laborer
 * @date 2024-10-16 15:34
 *
 */

    private JSONObject bindingAdd(SoSaleEntity rec) {
        JSONObject main = new JSONObject();
        main.put("formCode","formmain_0047");
//        main.put("loginName",rec.getUser_code());
        main.put("loginName","yuqh");
        main.put("rightId","-8974880151631871438.3094438342688065626");
        JSONArray dataList = new JSONArray();
        JSONObject dataListObj = new JSONObject();

        JSONObject masterTable = new JSONObject();
        JSONArray subTables = new JSONArray();
        dataListObj.put("masterTable", masterTable);
        masterTable.put("name","formmain_0047");
        JSONObject record = new JSONObject();
        record.put("id", UUID.randomUUID().getMostSignificantBits());
        JSONArray fields = new JSONArray();
        JSONObject field0001 = new JSONObject();//单据号
        field0001.put("showValue",rec.getBillCode());
        field0001.put("name","field0001");
        field0001.put("value",rec.getBillCode());
        fields.add(field0001);

        JSONObject field0004 = new JSONObject();//单据日期
        field0004.put("showValue",rec.getDapprovedate());
        field0004.put("name","field0004");
        field0004.put("value",rec.getDapprovedate());
        fields.add(field0004);

        JSONObject field0033 = new JSONObject();//合同编号
        field0033.put("showValue",rec.getBillCode());
        field0033.put("name","field0033");
        field0033.put("value",rec.getBillCode());
        fields.add(field0033);

        JSONObject field0014 = new JSONObject();//客户名称
        field0014.put("showValue",rec.getCustname());
        field0014.put("name","field0014");
        field0014.put("value",rec.getBillCode());
        fields.add(field0014);

        JSONObject field0045 = new JSONObject();//客户编码
        field0045.put("showValue",rec.getCustcode());
        field0045.put("name","field0045");
        field0045.put("value",rec.getCustcode());
        fields.add(field0045);

        JSONObject field0005 = new JSONObject();//销售组织
        field0005.put("showValue",rec.getVsalestruname());
        field0005.put("name","field0005");
        field0005.put("value",rec.getBillCode());
        fields.add(field0005);

        JSONObject field0006 = new JSONObject();//库存组织
        field0006.put("showValue",rec.getBodyname());
        field0006.put("name","field0006");
        field0006.put("value",rec.getBodyname());
        fields.add(field0006);

        JSONObject field0007 = new JSONObject();//收货单位
        field0007.put("showValue",rec.getShname());
        field0007.put("name","field0007");
        field0007.put("value",rec.getShname());
        fields.add(field0007);

        JSONObject field0009 = new JSONObject();//开票单位
        field0009.put("showValue",rec.getKpname());
        field0009.put("name","field0009");
        field0009.put("value",rec.getKpname());
        fields.add(field0009);

        JSONObject field0013 = new JSONObject();//业务流程
        field0013.put("showValue",rec.getBusiname());
        field0013.put("name","field0013");
        field0013.put("value",rec.getBusiname());
        fields.add(field0013);

        JSONObject field0018 = new JSONObject();//备注
        field0018.put("showValue",rec.getVnote());
        field0018.put("name","field0018");
        field0018.put("value",rec.getVnote());
        fields.add(field0018);

        record.put("fields",fields);
        masterTable.put("record",record);
        JSONArray records = new JSONArray();
        JSONObject subTablesObj = new JSONObject();

        if(CollectionUtils.isNotEmpty(rec.getSaleLineList())){
            rec.getSaleLineList().forEach(line->{
                JSONObject body = new JSONObject();
                body.put("id",UUID.randomUUID().getMostSignificantBits());
                JSONArray bodyDetails = new JSONArray();
                JSONObject field0020 = new JSONObject();//存货编号
                field0020.put("showValue",line.getInvcode());
                field0020.put("name","field0020");
                field0020.put("value",line.getInvcode());
                bodyDetails.add(field0020);
                JSONObject field0021 = new JSONObject();//存货名称
                field0021.put("showValue",line.getInvname());
                field0021.put("name","field0021");
                field0021.put("value",line.getInvname());
                bodyDetails.add(field0021);

                JSONObject field0022 = new JSONObject();//规格
                field0022.put("showValue",line.getInvspec());
                field0022.put("name","field0022");
                field0022.put("value",line.getInvspec());
                bodyDetails.add(field0022);

                JSONObject field0023 = new JSONObject();//型号
                field0023.put("showValue",line.getInvtype());
                field0023.put("name","field0023");
                field0023.put("value",line.getInvtype());
                bodyDetails.add(field0023);

                JSONObject field0024 = new JSONObject();//单位
                field0024.put("showValue",line.getMeasname());
                field0024.put("name","field0024");
                field0024.put("value",line.getMeasname());
                bodyDetails.add(field0024);

                JSONObject field0025 = new JSONObject();//数量
                field0025.put("showValue",line.getNnumber());
                field0025.put("name","field0025");
                field0025.put("value",line.getNnumber());
                bodyDetails.add(field0025);

                JSONObject field0026 = new JSONObject();//无税单价
                field0026.put("showValue",line.getNoriginalcurprice());
                field0026.put("name","field0026");
                field0026.put("value",line.getNoriginalcurprice());
                bodyDetails.add(field0026);

                JSONObject field0027 = new JSONObject();//税率
                field0027.put("showValue",line.getNtaxrate());
                field0027.put("name","field0027");
                field0027.put("value",line.getNtaxrate());
                bodyDetails.add(field0027);

                JSONObject field0028 = new JSONObject();//含税单价
                field0028.put("showValue",line.getNoriginalcurtaxprice());
                field0028.put("name","field0028");
                field0028.put("value",line.getNoriginalcurtaxprice());
                bodyDetails.add(field0028);

                JSONObject field0029 = new JSONObject();//无税金额
                field0029.put("showValue",line.getNoriginalcurmny());
                field0029.put("name","field0029");
                field0029.put("value",line.getNoriginalcurmny());
                bodyDetails.add(field0029);

                JSONObject field0030 = new JSONObject();//价税合计
                field0030.put("showValue",line.getNoriginalcursummny());
                field0030.put("name","field0030");
                field0030.put("value",line.getNoriginalcursummny());
                bodyDetails.add(field0030);

                JSONObject field0031 = new JSONObject();//是否赠品
                field0031.put("showValue",line.getBlargessflag());
                field0031.put("name","field0031");
                field0031.put("value",line.getBlargessflag());
                bodyDetails.add(field0031);

                JSONObject field0032 = new JSONObject();//合同编号
                field0032.put("showValue",line.getFrownote());
                field0032.put("name","field0032");
                field0032.put("value",line.getFrownote());
                bodyDetails.add(field0032);
                body.put("fields",bodyDetails);
            });
            subTablesObj.put("records",records);
            subTablesObj.put("name","formson_0048");
            subTables.add(subTablesObj);
        }
        dataListObj.put("subTables", subTables);
        dataList.add(dataListObj);
        main.put("dataList",dataList);
        return main;
    }
}
